/* analizzatore sintattico */

%{
#include <stdio.h>
#include <math.h>
#include <strings.h>
#include "h3calc-shared.h"	/* macro e funzioni condivise */
%}

/* Dichiarazioni dei token per BISON */

%left 	NUM		/* numero floating point */
%left	VAR		/* identificativo di variabile */
%left 	STR		/* stringa */
%left   PRINT PRINTLN INPUT
%left 	VARIABLES CONSTANTS FUNCTIONS HELP QUIT

%right 	'='		/* assegnamento */

%left   ','     /* concatena stringhe */

%left 	IMPLIES INV_IMPLIES /* operatori logici */
%left 	OR					/* operatori logici */
%left 	XOR					/* operatori logici */
%left 	AND					/* operatori logici */
%left 	NOT					/* operatori logici */

%left 	EQUALS NOT_EQUALS	/* operatori logici */
%left 	'<' '>'				/* operatori logici */

%left 	SHL SHR				/* shift */

%left 	'+' '-'		/* operatori aritmetici */
%left 	'*' '/' '%'	/* operatori aritmetici */

%left   '[' ':' '?' ']' /* if then else */

%left 	ABS		/* valore assoluto */
 		SQRT 	/* radice quadrata */
 		CBRT	/* radice cubica */
 		LN		/* logaritmo naturale */
 		LOG		/* logaritmo in base 10 */
 		CEIL	/* arrotondamento per eccesso */
 		FLOOR	/* arrotondamento per difetto */
 		ROUND	/* arrotondamento al pi vicino */
 		EXP		/* esponenziale */
		SIN		/* seno */
 		COS    	/* coseno */
 		TAN		/* tangente */
 		SINH    /* seno iperbolico */
 		COSH    /* coseno iperbolico */
 		TANH	/* tangente iperbolica */
 		ARCSIN  /* arcoseno */
 		ARCCOS  /* arcocoseno */
 		ARCTAN	/* arcotangente */

%right 	'^'    	/* elevamento a potenza  */
%left 	NEG     /* negazione (il segno meno) */
%left 	'!'		/* fattoriale */
%left '(' ')'	/* parentesi */



/* Definizione della grammatica */
%%
input:    /* stringa vuota */
        | input line
;

line:     '\n'				{   return 0; }
        | exp '\n'  		{ 	printf ("%.10g\n", $1.value); return 0;	}
        | VAR '=' exp 		{ 
        						dprintf ("assign: %s = %g\n", $1.name, $3.value); 
        						setvar($1.name, $3.value, FALSE);
        						//printf ("\n");
        					}
        | QUIT				{	return 2;	}
        | VARIABLES			{
        						printf("\n--- variables ---\n");
        						int num=showvars();
        						printf("--- %d found ---\n\n",num);
        					}
        | CONSTANTS			{
        						printf("\n--- constants ---\n");
        						int num=showconsts();
        						printf("--- %d found ---\n\n",num);
        					}
        | HELP				{
        						printf("\n--- help ---\n");

                                printf(  " Here are some examples of what you can do:\n\n");
                                printf(  " > variables                          ... lists user defined vars\n");
                                printf(  " > constants                          ... lists predefined constants\n");
                                printf(  " > functions                          ... lists predefined functions\n");
                                printf(  " > help                               ... shows this help screen\n");
                                printf(  " > quit                               ... quits\n");
                                printf(  " > ln(1+(4+2)*3)+sin(pi*e)            ... outputs the result\n");
                                printf(  " > pippo = 6+round(cos(3)*10)         ... defines a variable \n");
                                printf(  " > pippo                              ... outputs a variable's value\n");
                                printf(  " > pluto = [ pippo<0 ? pippo : 0 ]    ... if ... then ... else ...\n");
                                printf(  " > fast and small or not expensive    ... fuzzy logic (vars in range 0..1)\n");
                                printf(  "\n Examples of what you can do in the 'read from file' mode:\n\n");
                                printf(  " > print \"any string\"                 ... prints to screen a string\n");
                                printf(  " > println                            ... goes to the next row\n");
                                printf(  " > println \"string\",var,\"string\"      ... prints strings and variables\n");
                                printf(  " > pippo = input                      ... asks to the user for a value\n");
                                printf(  "------------\n\n");
        					}
       	| FUNCTIONS			{
        						printf("\n--- functions ---\n");
        						printf(  "abs and arccos arcsin arctan cbrt ceil cos cosh exp floor ln log\n");
								printf(  "not or round sin sinh sqrt tan tanh xor\n");
								printf(  "( ) + - * / %% [:?] ! ^ << >> < <= > >= == != => =<\n");
								printf(  "input print println\n");
								printf(  "quit exit help variables constants functions\n");
								printf(  "-----------------\n\n");
							}
	    | PRINT string		%prec PRINT		{	printf( "%s", $2.name);		} 
        | PRINTLN string	%prec PRINTLN	{ 	printf( "%s\n", $2.name); 	} 
        | PRINTLN			%prec PRINTLN	{	printf("\n");	} 
        
;

string:   STR				{   strcpynz($$.name,$1.name,MAX_VARIABLE_NAME_LENGTH);		}
        | VAR				{
        						struct variableinfo *v = getvar($1.name);
							   	if (v==NULL) {
							   		sprintf($$.name,"%.10g",0.0); 						
							   		printf("warning: variable \"%s\" undefined, assuming %s = 0\n",$1.name,$1.name); 
							   	} else {
							   		sprintf($$.name,"%.10g",v->value);
							   	}	
        					}
        | string ',' string {
        						strcpynz($$.name,$1.name,MAX_VARIABLE_NAME_LENGTH);
        						strcat($$.name,$3.name);
        					}
;

exp:      NUM                { $$.value = $1.value;  				dprintf("value: %g\n",$1.value);	}
		| VAR				 { 
							   	struct variableinfo *v = getvar($1.name);
							   	if (v==NULL) {
							   		$$.value = 0.0; 						
							   		printf("warning: variable \"%s\" undefined, assuming %s = 0\n",$1.name,$1.name); 
							   	} else {
							   		$$.value = v->value;
							   		dprintf("var: \"%s\" has value %g\n",$1.name,v->value); 
							   	}
						  	 }
		| INPUT				 {
								float temp;
								fscanf(stdin,"%g",&temp);
								$$.value=temp;
							 }
		| exp AND exp        { 
								if($1.value<$3.value) $$.value=$1.value; else $$.value=$3.value;
								dprintf("and: %g and %g = %g\n",$1.value,$3.value,$$.value);	
							 }
	    | exp OR exp         { 
								if($1.value>$3.value) $$.value=$1.value; else $$.value=$3.value;
								dprintf("or: %g or %g = %g\n",$1.value,$3.value,$$.value);	
							 }
		| exp XOR exp        { 
								double temp1 = ($1.value < 1.0-$3.value) ? $1.value : 1.0-$3.value ;
								double temp2 = (1.0-$1.value < $3.value) ? 1.0-$1.value : $3.value ;
								$$.value = (temp1 > temp2) ? temp1 : temp2;
								dprintf("xor: %g or %g = %g\n",$1.value,$3.value,$$.value);	
							 }
		| exp IMPLIES exp    { 
								$$.value = (1.0-$1.value > $3.value) ? 1.0-$1.value : $3.value;
								dprintf("=>: %g => %g = %g\n",$1.value,$3.value,$$.value);	
							 }
		| exp INV_IMPLIES exp { 
								$$.value = (1.0-$3.value > $1.value) ? 1.0-$3.value : $1.value;
								dprintf("=>: %g => %g = %g\n",$1.value,$3.value,$$.value);	
							 }
	    | NOT exp  %prec NOT { 
								$$.value=1.0 - $2.value; 
								dprintf("not: not %g = %g\n",$2.value,$$.value);	
							 }
		| exp SHL exp        { 
								$$.value=floor($1.value*pow(2,$3.value));
								dprintf("<<: %g << %g = %g\n",$1.value,$3.value,$$.value);	
							 }		
		| exp SHR exp        { 
								$$.value=floor($1.value/pow(2,$3.value));
								dprintf(">>: %g >> %g = %g\n",$1.value,$3.value,$$.value);	
							 }				 				 
		| exp '<' exp        { 
								if($1.value<$3.value) $$.value=1.0; else $$.value=0.0;
								dprintf("<: %g < %g = %g\n",$1.value,$3.value,$$.value);	
							 }
		| exp '>' exp        { 
								if($1.value>$3.value) $$.value=1.0; else $$.value=0.0;
								dprintf(">: %g > %g = %g\n",$1.value,$3.value,$$.value);	
							 }
		| exp '<' '=' exp    { 
								if($1.value<=$3.value) $$.value=1.0; else $$.value=0.0;
								dprintf("<=: %g <= %g = %g\n",$1.value,$3.value,$$.value);	
							 }
		| exp '>' '=' exp    { 
								if($1.value>=$3.value) $$.value=1.0; else $$.value=0.0;
								dprintf(">=: %g >= %g = %g\n",$1.value,$3.value,$$.value);	
							 }
		| exp EQUALS exp   %prec '<'   { 
								if($1.value==$3.value) $$.value=1.0; else $$.value=0.0;
								dprintf("==: %g == %g = %g\n",$1.value,$3.value,$$.value);	
							 }
	    | exp NOT_EQUALS exp    { 
								if($1.value!=$3.value) $$.value=1.0; else $$.value=0.0;
								dprintf("!=: %g != %g = %g\n",$1.value,$3.value,$$.value);	
							 }
        | exp '+' exp        { $$.value = $1.value + $3.value;		dprintf("add: %g + %g = %g\n",$1.value,$3.value,$$.value);	}
        | exp '-' exp        { $$.value = $1.value - $3.value; 		dprintf("sub: %g - %g = %g\n",$1.value,$3.value,$$.value);  }
        | exp '*' exp        { $$.value = $1.value * $3.value;		dprintf("mul: %g * %g = %g\n",$1.value,$3.value,$$.value);  }
        | exp '/' exp        { $$.value = $1.value / $3.value;		dprintf("div: %g / %g = %g\n",$1.value,$3.value,$$.value);  }
        | exp '%' exp        { $$.value = fmod($1.value,$3.value);		dprintf("mod: %g % %g = %g\n",$1.value,$3.value,$$.value);  }
        | '-' exp  %prec NEG { $$.value = -$2.value;        		dprintf("neg: - %g = %g\n",$2.value,$$.value);  }
        | exp '^' exp        { $$.value = pow($1.value,$3.value); 	dprintf("pow: %g ^ %g = %g\n",$1.value,$3.value,$$.value);  }
        | exp '!'  %prec '!' { 
        					   double difff=fabs( $1.value-(double)(rint( $1.value )) );
        					   if( difff < 0.000000001 ) {
        					   		$$.value=fatt(rint( $1.value ));
        		                    dprintf("fatt: %g ! = %g\n",$1.value,$$.value);  
        		               } else printf("*ERROR: %g !\n",$1.value);
                             }
        | '(' exp ')'        { $$.value = $2.value;         		dprintf("( ): ( %g )\n",$2);  }
        | '['  exp  '?' exp ':' exp ']' 	{
        									if($2.value >= 0.5) $$.value = $4.value;
        									else $$.value = $6.value;
        									dprintf("[ ]: [ %g ? %g : %g ] = %g\n",$2.value,$4.value,$6.value,$$.value); 
        								}
        | CEIL '(' exp ')'   { $$.value = ceil ( $3.value ); 		dprintf("ceil: ceil( %g ) = %g\n",$3.value,$$.value);  }
        | FLOOR '(' exp ')'  { $$.value = floor ( $3.value ); 		dprintf("floor: floor( %g ) = %g\n",$3.value,$$.value);  }
        | ROUND '(' exp ')'  { $$.value = rint ( $3.value ); 		dprintf("round: round( %g ) = %g\n",$3.value,$$.value);  }
        | EXP '(' exp ')'    { $$.value = exp ( $3.value ); 		dprintf("exp: exp( %g ) = %g\n",$3.value,$$.value);  }
        | ABS '(' exp ')'    { $$.value = fabs ( $3.value ); 		dprintf("abs: abs( %g ) = %g\n",$3.value,$$.value);  }
        | SIN '(' exp ')'    { $$.value = sin ( $3.value ); 		dprintf("sin: sin( %g ) = %g\n",$3.value,$$.value);  }
        | COS '(' exp ')'    { $$.value = cos ( $3.value ); 		dprintf("cos: cos( %g ) = %g\n",$3.value,$$.value);  }
        | TAN '(' exp ')'    { $$.value = tan ( $3.value ); 		dprintf("tan: tan( %g ) = %g\n",$3.value,$$.value);  }
        | SINH '(' exp ')'   { $$.value = sinh ( $3.value ); 		dprintf("sinh: sinh( %g ) = %g\n",$3.value,$$.value);  }
        | COSH '(' exp ')'   { $$.value = cosh ( $3.value ); 		dprintf("cosh: cosh( %g ) = %g\n",$3.value,$$.value);  }
        | TANH '(' exp ')'   { $$.value = tanh ( $3.value ); 		dprintf("tanh: tanh( %g ) = %g\n",$3.value,$$.value);  }
        | ARCSIN '(' exp ')' { $$.value = asin ( $3.value ); 		dprintf("arcsin: arcsin( %g ) = %g\n",$3.value,$$.value);  }
        | ARCCOS '(' exp ')' { $$.value = acos ( $3.value ); 		dprintf("arccos: arccos( %g ) = %g\n",$3.value,$$.value);  }
        | ARCTAN '(' exp ')' { $$.value = atan ( $3.value ); 		dprintf("arctan: arctan( %g ) = %g\n",$3.value,$$.value);  }
        | SQRT '(' exp ')'   { if ($3.value>0) { 
        							$$.value = sqrt ( $3.value ); 
        							dprintf("sqrt: sqrt( %g ) = %g\n",$3.value,$$.value); 
        					   } else {
        					   	 	printf("*ERROR: sqrt( %g )\n",$3.value);
        					   }
        					 }
        | CBRT '(' exp ')'   { if ($3.value>0) { 
        							$$.value = cbrt ( $3.value ); 
        							dprintf("cbrt: cbrt( %g ) = %g\n",$3.value,$$.value); 
        					   } else {
        					   	 	printf("*ERROR: cbrt( %g )\n",$3.value);
        					   }
        					 }
       	| LN '(' exp ')'     { if ($3.value>0) { 
        							$$.value = log ( $3.value ); 
        							dprintf("ln: ln( %g ) = %g\n",$3.value,$$.value); 
        					   } else {
        					   	 	printf("*ERROR: ln( %g )\n",$3.value);
        					   }
        					 }
       	| LOG '(' exp ')'    { if ($3.value>0) { 
        							$$.value = log10 ( $3.value ); 
        							dprintf("log: log( %g ) = %g\n",$3.value,$$.value); 
        					   } else {
        					   	 	printf("*ERROR: log( %g )\n",$3.value);
        					   }
        					 }
;
%%

int yyerror(char *s)  /* Chiamata da yyparse in caso di errore */
{
  printf("*ERROR: %s\n", s);
  fflush(stdin);
  return 1;
}
