/* analizzatore lessicale */

%{
#include <math.h> 			/* necessaria per utilizzare atof() */
#include <strings.h>	    /* necessaria per strlen() */
#include "h3calc-shared.h"  /* macro e funzioni condivise */
#include "h3calc-bison.h"   /* definisce i token usati in bison e la variabile yylval */
%}

DIGIT    [0-9]
ID       [a-z][a-z0-9_]*

%%

{DIGIT}+	{
	yylval.value=atof(yytext);
	dprintf( "integer: %s (%d)\n", yytext, atoi( yytext ) );
	return NUM;
}
	                                
{DIGIT}+"."{DIGIT}*	{
	yylval.value=atof(yytext);
    dprintf( "float: %s (%g)\n", yytext, yylval );
    return NUM;
}

"\""[^\"\n]*"\"" {
	char *temp = (char *)yytext;
	temp++;
	dprintf( "string: %s\n", yytext );
	strcpynz(yylval.name,temp,MAX_VARIABLE_NAME_LENGTH);
	yylval.name[strlen(yylval.name)-1]=0;	
	return STR;
}

"print" {
	dprintf( "token: %s\n", yytext );					
	return PRINT;
}

"println" {
	dprintf( "token: %s\n", yytext );					
	return PRINTLN;
}

"input" {
	dprintf( "token: %s\n", yytext );					
	return INPUT;
}

"quit"|"exit"   {
	dprintf( "token: %s\n", yytext );					
	return QUIT;
}

"help"   {
	dprintf( "token: %s\n", yytext );					
	return HELP;
}

"env"|"environment"|"var"|"vars"|"variables"|"variable"  {
	dprintf( "token: %s\n", yytext );					
	return VARIABLES;
}

"const"|"consts"|"constants"  {
	dprintf( "token: %s\n", yytext );					
	return CONSTANTS;
}

"functions"  {
	dprintf( "token: %s\n", yytext );					
	return FUNCTIONS;
}

"="|"+"|"-"|"*"|"/"|"%"|"^"|"!"|"?"|":"|"["|"]"|"<"|">"|","   {
	dprintf( "token: %s\n", yytext );
	return yytext[0];
}	

"or"   {
	dprintf( "operator: %s\n", yytext );					
	return OR;
}	

"and"   {
	dprintf( "operator: %s\n", yytext );					
	return AND;
}	

"not"   {
	dprintf( "operator: %s\n", yytext );					
	return NOT;
}

"xor"   {
	dprintf( "operator: %s\n", yytext );					
	return XOR;
}	

"=>"   {
	dprintf( "operator: %s\n", yytext );					
	return IMPLIES;
}	

"=<"   {
	dprintf( "operator: %s\n", yytext );					
	return INV_IMPLIES;
}	

"<<"   {
	dprintf( "operator: %s\n", yytext );					
	return SHL;
}	

">>"   {
	dprintf( "operator: %s\n", yytext );					
	return SHR;
}	


"=="   {
	dprintf( "operator: %s\n", yytext );					
	return EQUALS;
}	

"!="   {
	dprintf( "operator: %s\n", yytext );					
	return NOT_EQUALS;
}	
				  
"sin"   {
	dprintf( "operator: %s\n", yytext );					
	return SIN;
}	

"cos"   {
	dprintf( "operator: %s\n", yytext );					
	return COS;
}	

"tan"   {
	dprintf( "operator: %s\n", yytext );					
	return TAN;
}	

"sinh"   {
	dprintf( "operator: %s\n", yytext );					
	return SINH;
}	

"cosh"   {
	dprintf( "operator: %s\n", yytext );					
	return COSH;
}	

"tanh"   {
	dprintf( "operator: %s\n", yytext );					
	return TANH;
}	

"arcsin"   {
	dprintf( "operator: %s\n", yytext );					
	return ARCSIN;
}	

"arccos"   {
	dprintf( "operator: %s\n", yytext );					
	return ARCCOS;
}	

"arctan"   {
	dprintf( "operator: %s\n", yytext );					
	return ARCTAN;
}	

"sqrt"   {
	dprintf( "operator: %s\n", yytext );					
	return SQRT;
}	

"cbrt"   {
	dprintf( "operator: %s\n", yytext );					
	return CBRT;
}	

"ln"   {
	dprintf( "operator: %s\n", yytext );					
	return LN;
}	

"log"   {
	dprintf( "operator: %s\n", yytext );					
	return LOG;
}	

"ceil"   {
	dprintf( "operator: %s\n", yytext );					
	return CEIL;
}	

"floor"   {
	dprintf( "operator: %s\n", yytext );					
	return FLOOR;
}	

"round"   {
	dprintf( "operator: %s\n", yytext );					
	return ROUND;
}	

"exp"   {
	dprintf( "operator: %s\n", yytext );					
	return EXP;
}	

"abs"   {
	dprintf( "operator: %s\n", yytext );					
	return ABS;
}	

				  
"("|")"   {
	dprintf( "operator: %s\n", yytext );
	return yytext[0];
}
		  
{ID}	{
	dprintf( "identifier: %s\n", yytext );
	strcpynz(yylval.name,yytext,MAX_VARIABLE_NAME_LENGTH);
	return VAR;
}
		
"\n"	{
	dprintf( "newline\n" );
	return yytext[0];
}		

"{"[^}\n]*"}"	/* elimina i commenti tra parentesi graffe */

[ \t]+          /* elimina gli spazi e i tab */

.           	dprintf( "Unrecognized character: %s\n", yytext );

%%

int yywrap() /* per gestire files di input multipli, in questo caso quindi non serve */
{
   return 1;
}
