/* funzioni condivise */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include "h3calc-shared.h"

struct variableinfo *firstVar=NULL;

void strcpynz(char *dest, char *src,int maxlen)
/* copia al massimo maxlen caratteri dalla stringa src alla stringa dest,
   garantisce che la stringa dest risultante sia zero-terminata, 
   quindi la dimensione di dest deve essere almeno maxlen */
{
	int i=0;
	while ((i<maxlen) && (src[i]!=0)) {
		dest[i]=src[i];
		i++;
	}
	dest[i]=0;
}

double fatt(double x) 
/* calcola il fattoriale di x */
{
    double i,f=1.0;

    for(i=2.0; i<=x; i=i+1.0) f*=i;
   
    return f;
}

int setvar(char* name,double value,int isconstant)
{
	struct variableinfo *tempVar=firstVar;
	int found = FALSE;
	
	while(tempVar!=NULL) {
		if (strcmp(name,tempVar->name)==0) {
			found = TRUE;
			break;
		}
		tempVar=tempVar->next;
	}
	
	if(found && tempVar->isconstant) {
		printf("*ERROR: assignement to a constant (\"%s\")\n",name);
		return FALSE;
	}
		
	if(found && !tempVar->isconstant) {
		tempVar->value=value;
		tempVar->isconstant=isconstant;
		return TRUE;
	}
	
	/* se si  giunti a questo punto significa che si sta definendo una nuova variabile mai dichiarata prima */
	/* quindi va aggiunta in testa alla lista */
	
	tempVar=(struct variableinfo *)malloc(sizeof(struct variableinfo));
	strcpynz(tempVar->name,name,MAX_VARIABLE_NAME_LENGTH);
	tempVar->value=value;
	tempVar->isconstant=isconstant;
	tempVar->next=firstVar;
	firstVar=tempVar;
	
	return TRUE;
}

struct variableinfo *getvar(char* name)
{
    struct variableinfo *tempVar=firstVar;
	int found = FALSE;
	
	while(tempVar!=NULL) {
		if (strcmp(name,tempVar->name)==0) {
			found = TRUE;
			break;
		}
		tempVar=tempVar->next;
	}
	
	if (found) return tempVar;
	
	return NULL;
}

int showvars()
{
    struct variableinfo *tempVar=firstVar;
    int count=0;
	
	while(tempVar!=NULL) {
		if (!tempVar->isconstant) {
			printf("%s = %.10g\n",tempVar->name,tempVar->value);
			count++;
		}
		tempVar=tempVar->next;	
	}
	
	return count;
}

int showconsts()
{
    struct variableinfo *tempVar=firstVar;
    int count=0;
	
	while(tempVar!=NULL) {
		if (tempVar->isconstant) {
			printf("%s = %.10g\n",tempVar->name,tempVar->value);
			count++;
		}
		tempVar=tempVar->next;	
	}
	
	return count;
}
